/*
 * Image plugin to VDR (C++)
 *
 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <signal.h>
#include <wait.h>

#include <linux/dvb/video.h>
#include <vdr/device.h>
#include <vdr/status.h>
#include <vdr/tools.h>
#include <vdr/thread.h>

#include "player-image.h"
#include "control-image.h"
#include "setup-image.h"
#include "data-image.h"



const char *g_szConvertScript = "convert.sh";
const char *g_szZoomScript    = "convert_zoom.sh";
const char *g_szJumpScript    = "convert_jump.sh";


//----------cImagePlayer-------------




cImagePlayer::cImagePlayer(cSlideShow *pCurSlideShow)
: cPlayer(
#if VDRVERSNUM >= 10308
 (ImageSetup.m_bLiveAudio != 0)?pmVideoOnly:
#endif
 pmAudioVideo
 )
, m_pCurSlideShow(pCurSlideShow)
{
  m_bIsConvertRunning = false;
  m_bIsShowImageRunning = false;
  m_nCurrentImage = 0;
  m_nTotalImages = 0;
  m_bActive = true;
  
  m_fdinPipe[0] = m_fdinPipe[1] = -1;
  m_fdOutPipe[0] = m_fdOutPipe[1] = -1;
  m_proPID = -1;

  if(m_pCurSlideShow)
  {
    m_nTotalImages = m_pCurSlideShow->Count();
    // get real Index of FirstImage
    unsigned int i = 0;
    bool bFound = false;
    if(m_pCurSlideShow->FirstImage()) {
      cImage* p = m_pCurSlideShow->cList < cImage >::First();
      for (;p && i < m_nTotalImages; ++i)
      {
        // find out first image
        if(0 == strcmp(::basename(p->Name()),m_pCurSlideShow->FirstImage()))
        {
          m_pCurImage = p;
          bFound = true;
          break;
        }
        p = m_pCurSlideShow->cList < cImage >::Next(p);
      }
    }
    if(bFound) {
      m_nCurrentImage = i+1;
    }
    else {
      m_pCurImage = m_pCurSlideShow->cList < cImage >::First();
      m_nCurrentImage = 1;
    }
  }
}

bool cImagePlayer::GetIndex(int &nCurrent, int &nTotal, bool /*bSnapToIFrame*/)
{
  // Notify other status-plugins with one picture as one second
  nCurrent = m_nCurrentImage; 
  // Notify other status-plugins count of picture as totalcount of seconds
  nTotal = m_nTotalImages;

  return true;
}

cImagePlayer::~cImagePlayer()
{
  kill(m_proPID, SIGTERM);
  waitpid(m_proPID, 0, 0);
  ClosePipe();
  Detach();
  // Remove Slideshow
  if(m_pCurSlideShow) {
    delete m_pCurSlideShow;
    m_pCurSlideShow = 0;
  }
}


void cImagePlayer::Activate(bool bOn)
{
  if(bOn)
  {
    if(m_pCurSlideShow && m_pCurImage)
        Convert(m_pCurImage->Name(), "");
  }
  else if(m_bActive) {
    m_bActive = false;
  }
}



void cImagePlayer::ClosePipe(void)
{
  unsigned int i;
  for (i = 0; i < sizeof(m_fdinPipe)/sizeof(m_fdinPipe[0]); ++i) {
    if(m_fdinPipe[i] >= 0) {
      close(m_fdinPipe[i]);
      m_fdinPipe[i] = -1;
    }
  }
  for (i = 0; i < sizeof(m_fdOutPipe)/sizeof(m_fdOutPipe[0]); ++i) {
    if(m_fdOutPipe[i] >= 0) {
      close(m_fdOutPipe[i]);
      m_fdOutPipe[i] = -1;
    }
  }    
}



bool cImagePlayer::Play(char** szShowImageError)
{
  if(!m_pCurImage)  // No image present, stop Plugin
      return false;
  if(!m_bIsConvertRunning)
  {
    ShowImage(szShowImageError);
  }
  return true;
}



bool cImagePlayer::NextImage(int nOffset)
{
  if(m_pCurImage && m_nCurrentImage < m_nTotalImages) {
    cImage *newcurr;
    for (int i = 0; (i < nOffset) && (m_nCurrentImage < m_nTotalImages); ++i) {
      newcurr = m_pCurSlideShow->cList < cImage >::Next(m_pCurImage);
      m_pCurImage = newcurr;
      ++m_nCurrentImage;
    }
    return m_pCurImage != NULL;
  } 
  else if(ImageSetup.AutoRepeat) {
    cImage *newcurr;
    
    newcurr = m_pCurSlideShow->cList < cImage >::First();
    m_pCurImage = newcurr;
    m_nCurrentImage = 1;
    
    return m_pCurImage != NULL;
  }
  return false;
}



bool cImagePlayer::PrevImage(int nOffset)
{
  if(m_pCurImage && m_nCurrentImage > 1) {
    cImage *newcurr;

    for (int i = 0; (i < nOffset) && (m_nCurrentImage > 1); ++i) {
        newcurr = m_pCurSlideShow->cList < cImage >::Prev(m_pCurImage);
        m_pCurImage = newcurr;
        --m_nCurrentImage;
    }
    return m_pCurImage != NULL;
  }
  else if(ImageSetup.AutoRepeat) {
    cImage *newcurr;

    newcurr = m_pCurSlideShow->cList < cImage >::Last();
    m_pCurImage = newcurr;
    m_nCurrentImage = m_nTotalImages;
      return m_pCurImage != NULL;
  }
  return false;
}


bool cImagePlayer::GotoImage(unsigned int nNewPictureIndex)
{
  if(m_pCurImage && nNewPictureIndex > 0 && nNewPictureIndex < m_nTotalImages) {
    cImage *newcurr;
    newcurr = m_pCurSlideShow->cList < cImage >::First();
    m_pCurImage = newcurr;
    m_nCurrentImage = 0;
    for (unsigned int i = 0; i < nNewPictureIndex; ++i) {
      newcurr = m_pCurSlideShow->cList < cImage >::Next(m_pCurImage);
      m_pCurImage = newcurr;
      ++m_nCurrentImage;
    }
    return m_pCurImage != NULL;
	}
  return false;
}

void cImagePlayer::IncSlideTime(void)
{
  if(ImageSetup.SSsec < 300) {
	    ImageSetup.SSsec++;
	}
}

void cImagePlayer::DecSlideTime(void)
{
  if(ImageSetup.SSsec > 1) {
	    ImageSetup.SSsec--;
	}
}

bool cImagePlayer::ChangeImage(const char *szChange)
{
  if(szChange 
    && m_pCurImage) {
    return Convert(m_pCurImage->Name(), szChange);
  }
  return false;
}

bool cImagePlayer::Convert(const char *szFileName, const char *szChange)
{
  m_bIsConvertRunning = true;
  ClosePipe();
  m_proPID = fork();
  if(m_proPID == -1)
	{
    esyslog("image: convert %s failed: (%d) %s", szFileName, errno, strerror(errno));
    return false;
	}
  if(m_proPID == 0)
  {
    dsyslog("image: script startet (pid=%d)", getpid());
    char szcmd[64 + PATH_MAX * 2];
    snprintf(szcmd, sizeof(szcmd), "%s \"%s\" \"%s\" %s", g_szConvertScript,
      szFileName, ImageSetup.TempDir, szChange ? szChange : "");
    
    ImageSetup.SetEnv();

    execle("/bin/sh", "sh", "-c", szcmd, 0, environ);
    esyslog("image: exec failed for %s: (%d) %s", szcmd, errno,
      strerror(errno));
    exit(127);
  }
  return true;
}

bool cImagePlayer::ConvertZoom(const char *szFileName, const char *szChange, int xRes,
			       int yRes, int xPos, int yPos)
{
  m_bIsConvertRunning = true;
  ClosePipe();
  m_proPID = fork();
  if(m_proPID == -1)
  {
    esyslog("image: convert %s failed: (%d) %s", szFileName, errno, strerror(errno));
    return false;
  }
  if(m_proPID == 0)
  {
    dsyslog("image: script startet (pid=%d)", getpid());
    static char szcmd[128 + PATH_MAX * 2];
    snprintf(szcmd, sizeof(szcmd),  "%s \"%s\" \"%s\" %s %d %d %d %d",
      g_szZoomScript, szFileName, ImageSetup.TempDir,
      szChange ? szChange : "Original", xRes, yRes, xPos, yPos);
    dsyslog("image: %s", szcmd);
  
    ImageSetup.SetEnv();
  
    execle("/bin/sh", "sh", "-c", szcmd, 0, environ);
    esyslog("image: exec failed for %s: (%d) %s", szcmd, errno,
      strerror(errno));
    exit(127);
  }
  return true;
}

bool cImagePlayer::ConvertJump(int nOffset)
{
  const unsigned int MAX_BILDER = 9;
  static char FileName[MAX_BILDER][PATH_MAX];
  unsigned int i;
  cImage *newcurr;
  int nBilder = 0;

  m_bIsConvertRunning = true;

  for (i = 0; i < MAX_BILDER; ++i)
    memset(FileName[i], 0, PATH_MAX);
  
  if(m_pCurImage && m_nCurrentImage > 0) {
  
	  int nJumpFirst = m_nCurrentImage + nOffset;

      if(nJumpFirst > ((int)(m_nTotalImages - MAX_BILDER + 1)))
		  nJumpFirst = ((int)(m_nTotalImages - MAX_BILDER + 1));
	  if(nJumpFirst < 1)
		  nJumpFirst = 1;
	  
	  m_nCurrentImage = nJumpFirst;
      if(m_nCurrentImage > m_nTotalImages)
		  m_nCurrentImage = m_nTotalImages;
	  if(m_nCurrentImage < 1)
		  m_nCurrentImage = 1;

    newcurr = m_pCurSlideShow->cList < cImage >::First();
    for (int n = 1; n < nJumpFirst && newcurr; ++n) 
      newcurr = m_pCurSlideShow->cList < cImage >::Next(newcurr);
	for (i = 0; (i < MAX_BILDER) && ((nJumpFirst + i) <= m_nTotalImages) && newcurr; ++i,++nBilder) {

      strncpy(FileName[i], newcurr->Name(), sizeof(FileName[0]));
	  dsyslog("image: File%d: %s", i, newcurr->Name());
	  if((nJumpFirst + i) == m_nCurrentImage)
		   m_pCurImage = newcurr;

	  newcurr = m_pCurSlideShow->cList < cImage >::Next(newcurr);
	}
  }
  
  ClosePipe();
  m_proPID = fork();

  if(m_proPID == -1) {
    esyslog("image: convertindex failed: (%d) %s", errno, strerror(errno));
    return false;
  }

  if(m_proPID == 0) {
    dsyslog("image: script startet (pid=%d)", getpid());
      static char szcmd[PATH_MAX * (MAX_BILDER + 3)];
    snprintf(szcmd, sizeof(szcmd),
       "%s \"%s\" %s %s \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
       g_szJumpScript, ImageSetup.TempDir,
       (nBilder > 6) ? "3x3" : ((nBilder > 3) ? "3x2" : "3x1"),
       "256x192", FileName[0], FileName[1], FileName[2],
       FileName[3], FileName[4], FileName[5], FileName[6],
       FileName[7], FileName[8]);
    dsyslog("%s", szcmd);
  
    ImageSetup.SetEnv();
    execle("/bin/sh", "sh", "-c", szcmd, 0, environ);
    esyslog("image: exec failed for %s: (%d) %s", szcmd, errno,
      strerror(errno));
    exit(127);
  }
  return true;
}



void cImagePlayer::ShowImage(char** szShowImageError)
{
  if(!m_bIsShowImageRunning)
	{
    m_bIsShowImageRunning = true;

    uchar *buffer;
    int filefd;
    ssize_t nRead;

    struct stat st;
    char szFullPathName[PATH_MAX];
    const char* szFile = "image.mpg" ;

    struct video_still_picture sp;
    snprintf(szFullPathName, sizeof(szFullPathName), 
      "%s/%s", ImageSetup.TempDir,szFile);

    if((filefd = open(szFullPathName, O_RDONLY)) >= 0) {
      dsyslog("file %s opened",szFullPathName);
      fstat(filefd, &st);
      
      if(st.st_size > 0) {
        sp.iFrame = (char *)malloc(st.st_size);
        sp.size = st.st_size;
  
        if(sp.iFrame) {
          nRead = read(filefd, sp.iFrame, sp.size);
          dsyslog("image: %d bytes fread from %s", nRead, szFullPathName);
  
          buffer = (uchar *) sp.iFrame;
  
          if(ImageSetup.UseDeviceStillPicture){
            DeviceStillPicture(buffer, sp.size);
          }
          else  {
            for (int i = 1; i <= 25; ++i)
              send_pes_packet(buffer, sp.size, i);
          }
    
          free(sp.iFrame);
        }
        else {
          asprintf(szShowImageError,"alloc of %d bytes failed",sp.size);
          esyslog("image: %s",*szShowImageError);
        }
      }
      else {
        asprintf(szShowImageError,"Can't read converted file %s it has zero size", szFile);
        esyslog("image: Can't read converted file %s it has zero size", szFullPathName);
      }
      close(filefd);
    }
    else {
      asprintf(szShowImageError,"Read converted file %s failed. (%d) %s", szFile ,errno, strerror(errno));
      esyslog("image: Read converted file %s failed. (%d) %s", szFullPathName ,errno, strerror(errno));
    }
    m_bIsShowImageRunning = false;
  }
}
  


void cImagePlayer::send_pes_packet(unsigned char *data, int len, int timestamp)
{
  const unsigned int PES_MAX_SIZE=2048;
  unsigned int ptslen = timestamp ? 5 : 1;
  static unsigned char pes_header[PES_MAX_SIZE];

  pes_header[0] = pes_header[1] = 0;
  pes_header[2] = 1;
  pes_header[3] = 0xe0;

  while(len > 0)
  {
    unsigned int payload_size = len;
    if(6 + ptslen + payload_size > PES_MAX_SIZE)
  		payload_size = PES_MAX_SIZE - (6 + ptslen);

    pes_header[4] = (ptslen + payload_size) >> 8;
    pes_header[5] = (ptslen + payload_size) & 255;

    if(ptslen == 5) {
      int x;
      x = (0x02 << 4) | (((timestamp >> 30) & 0x07) << 1) | 1;
      pes_header[8] = x;
      x = ((((timestamp >> 15) & 0x7fff) << 1) | 1);
      pes_header[7] = x >> 8;
      pes_header[8] = x & 255;
      x = ((((timestamp) & 0x7fff) < 1) | 1);
      pes_header[9] = x >> 8;
      pes_header[10] = x & 255;
    } 
    else  {
      pes_header[6] = 0x0f;
    }

    memcpy(&pes_header[6 + ptslen], data, payload_size);
    PlayVideo(pes_header, 6 + ptslen + payload_size);

    len -= payload_size;
    data += payload_size;
    ptslen = 1;
  }
}

bool cImagePlayer::Active(char** szShowImageError) 
{
  if(!m_bActive)
    return false; 
  if(!IsConvertRunning())
    return true; 

  waitpid(m_proPID, 0, WNOHANG);
  if(kill(m_proPID, 0) < 0 
    && errno == ESRCH)
  {
    ConvertRunning(false);
    return Play(szShowImageError);
  }
  return true;
}

const char* cImagePlayer::FileName(void) const
{ 
  return m_pCurImage?m_pCurImage->Name():NULL;
}
