/*
 * Image plugin to VDR (C++)
 *
 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___DVB_IMAGE_H
#define ___DVB_IMAGE_H

#include <vdr/player.h>


class cImage;
class cSlideShow;    
    
class cImagePlayer
: public cPlayer 
{
    
  /** Is the Player active */
  bool                        m_bActive;
  /** The Status of running ShowImage  */
  volatile bool               m_bIsShowImageRunning;
  /** The Status of running convert-script  */
  volatile bool               m_bIsConvertRunning;
  /** The current active filelist with the slide*/
  cSlideShow                  *m_pCurSlideShow;
  /** Current number of Image in slide */  
  unsigned int                m_nCurrentImage;
  /** Total number of Images in slide */  
  unsigned int                m_nTotalImages;
  /** Reference to current Image of this player  */
  cImage                      *m_pCurImage;

  int m_proPID;
  int m_fdinPipe[2];
  int m_fdOutPipe[2];

private:
  void ShowImage(char** szShowImageError);
  void ClosePipe(void);
  void send_pes_packet(unsigned char *data, int len, int timestamp);
protected:
  virtual void Activate(bool On);
public:

  cImagePlayer(cSlideShow *pCurSlideShow);
  virtual ~ cImagePlayer();

  bool Active(char** szShowImageError);
	bool Play(char** szShowImageError);
  bool NextImage(int Step);
  bool PrevImage(int Step);
  bool GotoImage(unsigned int nNewPictureIndex);
  
  /** Set the Status of running convert-script */
  void ConvertRunning(bool bIs)   { m_bIsConvertRunning = bIs;  }
  /** Deliver the Status of running convert-script */
  bool IsConvertRunning() const   { return m_bIsConvertRunning; }
  /** Deliver the current number of viewed Image */
  int ImageCurrent(void) const    { return m_nCurrentImage;     }
  /** Deliver the total number of viewed Image */
  int ImageTotal(void) const      { return m_nTotalImages;      }
  /** Deliver the filename from the current number of viewed Image */
  const char* FileName(void) const;
  
  void IncSlideTime(void);
  void DecSlideTime(void);
  bool ChangeImage(const char *szChange);
  bool Convert(const char *filename, const char *szChange);
  bool ConvertZoom(const char *filename, const char *szChange, int xRes, int yRes,
                   int xPos, int yPos);
  bool ConvertJump(int Step);
  
  /** Returns the current and total frame index, optionally snapped to the
  nearest I-frame.*/
  virtual bool GetIndex(int &Current, int &Total, bool SnapToIFrame);
};

#endif				//__DVB_IMAGE_H
