#!/bin/bash
###########################################################
# convert.sh
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
#
# History:
# 2004-06-01 a.brachold - remove some dead staff 
# 03/2004    Andreas Holzhammer: implemented non ImageMagick stuff, now you can do it without. 
#            also added some cache for zooming, reworked most of the scripts...
#            
# 02/2004    ok: removed "identify" from imagemagic for (much) slower pixel size detection by decoding 
#            with verbosity, optional see USE_IDENTIFY var, generate background for scaling with netpbm tools
# 02/2004    ok: added corrected pixel -> acpect calc. between 1.25->1.33 asp no black bars, 
#            rest is properly calculated and black borders are added (for 16/9 -> replace 125 with 170 and 133 with 178)
# 01/2004    Interpohl@vdr-portal.de reworked to suit the "new" image plugin
# 11/2003    ok: added cache browser stuff, reworked config, need identify from imagemagick againe, we need the pixes sizes, we can rotate now
# 16.05.2003 ok: removed "identify" replaced by file -i -L -b
#            removed image pixel detection, since using pnmcut we always generate 704x576
# 02-04.2003 ok (Onno Kreuzinger): reworked to use only mpegtools and pnm tools, no imagemagic/mpeg2encode 
#            needed only old identify used from X11 sw.
#
# 31.01.03 0.0.7 KTB // basename 
#			thanks to Andreas Kool
# 31.01.03 0.0.6 KTB // Rotation
# 28.01.03 0.0.5 KTB // 
# 27.01.03 0.0.4 KTB // DEBUG
# 27.01.03 0.0.3 KTB // caching 
# 23.01.03 0.0.2 KTB // geometry 
# 22.01.03 0.0.1 KTB // Initial Version
###########################################################
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"

INC_DIR=`dirname $0`
[ ! -e ${INC_DIR}/convert_functions.sh ] && echo "Can't find include file convert_functions.sh" && exit 1
. ${INC_DIR}/convert_functions.sh

if test "$1" == ""; then echo "Error, no parameters given!";exit 1;else IMAGEFILE="$1";fi
if test "$2" == ""; then CONVERT_TEMPDIR="/tmp/image"; else CONVERT_TEMPDIR=$2; fi ; 
if test "$2" == "nocopy"; then NOCOPY=1;CONVERT_TEMPDIR="$1";IMAGEFILE="$3";else NOCOPY=0; fi

convert_init "$CONVERT_TEMPDIR" "$CALLED"

DIR=$(dirname "$IMAGEFILE")
FILE=$(basename "$IMAGEFILE")
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
NEWFILE="${NEWDIR}/$FILE.mpg"
CMDFILE="${NEWDIR}/$FILE.cnv"
PARFILE="${NEWDIR}/$FILE.par"
logit "IMAGEFILE: $IMAGEFILE";
logit "CONVERT_MPEGFILE: $NEWFILE";
mkdir -p "$NEWDIR"

if test "$3" == "Right"; then ROTATE="YES"; FLIP="pnmflip -rotate270"; fi
if test "$3" == "Left";  then ROTATE="YES"; FLIP="pnmflip -rotate90";  fi
if test "$3" == "Original"; then ORIGINAL="YES"; fi
if test "$3" == "nocopy"; then NOCOPY=1; fi
echo "$3" > "$CMDFILE"

if test -f "$NEWFILE" && test "$ROTATE" == "YES" || test "$ORIGINAL" == "YES"; then rm -f "$NEWFILE"; fi

# mpeg2 still picture viewer hack
getpnmtool "$IMAGEFILE"
if [ $FILE_DETECT = "video/mpeg" ] ; then
   logit "MAIN: simply copying mpeg2 still"
   cp "$IMAGEFILE" "$NEWFILE"
fi

# This is needed for zooming. getresolution is cheap, not the whole file is converted
getresolution "$IMAGEFILE"
echo "$X_RES_TMP" >"$PARFILE"
echo "$Y_RES_TMP">>"$PARFILE"

#if mpg does not exist in cache
if [ ! -f "$NEWFILE" ] ; then
   [ ! -d "$NEWDIR" ] && mkdir -p -v "$NEWDIR" | tee -a "$CONVERT_LOGFILE"
   convert_imagetompeg "$IMAGEFILE" "$NEWFILE" "$FLIP"
else
   logit "$NEWFILE exists! convert not required, initialising data"
fi


if [ ! -f "$NEWFILE" ] ; then 
  logit "$DATE: ERROR Convert Image-to-MPEG2-STILL not successful for: $IMAGEFILE"; 
   cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg $CONVERT_MPEGFILE" &>/dev/null
else
  logit "$DATE: successfull ...";

   #kind a dirty break if we dont want to copy resulting file
   # but pregenrate a file (e.g. using at or cron
   [ $NOCOPY -eq 1 ] && exit 0

   if [ $(wc -c "$NEWFILE"|sed 's/^ *//'|cut -f1 -d" ") -gt 100 ] ; then
      logit "doing the copy to $CONVERT_MPEGFILE from $NEWFILE"
      rm -f "$CONVERT_MPEGFILE"
      cp -f "$NEWFILE" "$CONVERT_MPEGFILE" | tee -a "$CONVERT_LOGFILE"
   else
      logit "Error generated mpeg is smaller than 100 byte, so not used!!"
      cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg" "$CONVERT_MPEGFILE" &>/dev/null
   fi
fi
