#!/bin/bash
###########################################################
# convert_jump.sh
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
#
###########################################################
#set -vx
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"

INC_DIR=`dirname $0`
[ ! -e ${INC_DIR}/convert_functions.sh ] && echo "Can't find include file convert_functions.sh" && exit 1
. ${INC_DIR}/convert_functions.sh

CONVERT_TEMPDIR="${1:-"/tmp/image"}"
convert_init $CONVERT_TEMPDIR $CALLED

DIR=$(dirname "$4")
FILE=$(basename "$4")
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
mkdir -p "$NEWDIR"
NEWFILE="${NEWDIR}/${FILE}Z.mpg"
JUMPFILE="${NEWDIR}/JUMP.pnm"
logit "FILE: $FILE"
logit "SRCFILE1: $4"
logit "SRCFILE2: $5"
logit "SRCFILE3: $6"
logit "SRCFILE4: $7"
logit "SRCFILE5: $8"
logit "SRCFILE6: $9"
logit "SRCFILE7: ${10}"
logit "SRCFILE8: ${11}"
logit "SRCFILE9: ${12}"
logit "JUMPFILE: $JUMPFILE"

if [ "z$USE_MONTAGE" = "ztrue"  ] ; then
   $MONTAGE -tile "$2" -geometry "$3" -background black "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" "$$.jpg"
   logit '$MONTAGE -tile $2 -geometry $3 $4 $5 $6 $7 $8 $9 ${10} ${11} ${12} $$.jpg'
   jpegtopnm "$$.jpg" > $JUMPFILE
   rm "$$.jpg"
else
   index=1
   for i in "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" ; do
      if [ -f "$i" ]; then
         getpnmtool "$i"
         $TO_PNM "$i" | pnmscale -xysize 256 192 |\
            pnmcomp -align=center -valign=middle - "$DUMMY_TILE" > "${CONVERT_TEMPDIR}/${index}"
      else
         cp "$DUMMY_TILE" "${CONVERT_TEMPDIR}/${index}"
      fi
   let index++
   done

   cd "$CONVERT_TEMPDIR"
   if [ "$SHOW_NUMBERS" = "ztrue" ] ; then
      pnmindex -size 217 -noquant -across 3 1 2 3 4 5 6 7 8 9 > $JUMPFILE
   else
      pnmcat -black -leftright 1 2 3 > row1
      pnmcat -black -leftright 4 5 6 > row2
      pnmcat -black -leftright 7 8 9 > row3
      pnmcat -black -topbottom row1 row2 row3 > $JUMPFILE
      rm row1 row2 row3
   fi
   rm 1 2 3 4 5 6 7 8 9
fi

# $JUMPFILE now contains the 3x3 matrix in pnm format.
# TODO: Make sure resolution is correct 


# This is needed for aspect and rotation
getresolution "$JUMPFILE"


[ ! -d "$NEWDIR" ] && mkdir -p -v "$NEWDIR" | tee -a $CONVERT_LOGFILE

convert_imagetompeg "$JUMPFILE" "$NEWFILE" "$FLIP"

# NEWFILE contains the 3x3 matrix file in mpeg
if [ ! -f "$NEWFILE" ] ; then 
   logit "$DATE: ERROR Convert Image-to-MPEG2-STILL not successfull for : $IMAGEFILE"; 
   cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg" "$CONVERT_MPEGFILE" &>/dev/null
else
   logit "$DATE: successfull ...";

   if [ $(wc -c "$NEWFILE"|sed 's/^ *//'|cut -f1 -d" ") -gt 100 ] ; then
      logit "doing the copy to $CONVERT_MPEGFILE from $NEWFILE"
      rm -f "$CONVERT_MPEGFILE"
      cp -f "$NEWFILE" "$CONVERT_MPEGFILE" | tee -a "$CONVERT_LOGFILE"
   else
      logit "Error generated mpeg is smaller than 100 byte, so not used!!"
      cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg" "$CONVERT_MPEGFILE" &>/dev/null
   fi
fi

rm -f "$JUMPFILE"
