#!/bin/bash
###########################################################
# convert_zoom.sh
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
# for best and fastes extra tools it's still suggested to use ImageMagick
#
###########################################################
#set -vx
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"

if [ $# -ne 7 ] ; then
   echo "Error, wrong number of arguments!"
   exit 1
fi

INC_DIR=`dirname $0`
if [ ! -e ${INC_DIR}/convert_functions.sh ] ; then
   echo "Can't find include file convert_functions.sh"
   exit 1
fi
. ${INC_DIR}/convert_functions.sh

SRCFILE="$1"
CONVERT_TEMPDIR="${2:-"/tmp/image"}"

# set defaults
convert_init $CONVERT_TEMPDIR "$CALLED"

if test "$3" == "Right"; then ROTATE="YES"; FLIP="pnmflip -rotate270"; fi
if test "$3" == "Left";  then ROTATE="YES"; FLIP="pnmflip -rotate90";  fi
if test "$3" == "Original"; then ORIGINAL="YES"; fi

X_RES_ZOOM="$4"
Y_RES_ZOOM="$5"
X_POS_ZOOM="$6"
Y_POS_ZOOM="$7"

DIR=$(dirname "$SRCFILE")
FILE=$(basename "$SRCFILE")
logit "FILE: $FILE"
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
NEWFILE="${NEWDIR}/${FILE}Z.mpg"

ZOOMFILE="${NEWDIR}/${FILE}Z.jpg"
logit "SRCFILE : $SRCFILE"
logit "ZOOMFILE: $ZOOMFILE"
logit "X_RES_ZOOM: $X_RES_ZOOM"
logit "Y_RES_ZOOM: $Y_RES_ZOOM"
logit "X_POS_ZOOM: $X_POS_ZOOM"
logit "Y_POS_ZOOM: $Y_POS_ZOOM"

if [ "z$USE_CONVERT" = "ztrue" ] ; then
   #using ImageMagic convert
   $CONVERT -crop "$X_RES_ZOOM"x"$Y_RES_ZOOM"+"$X_POS_ZOOM"+"$Y_POS_ZOOM" "$SRCFILE" "$ZOOMFILE"
   logit "$CONVERT -crop ${X_RES_ZOOM}x${Y_RES_ZOOM}+${X_POS_ZOOM}+${Y_POS_ZOOM} $SRCFILE $ZOOMFILE"
else
   #using netpbm pnmcut
   logit "using netpbm tools for zoom"
   ZOOMFILE="${NEWDIR}/${FILE}Z.pnm"
   getpnmtool "$SRCFILE"

   if [ "z$USE_ZOOMCACHE" = "ztrue" ] ; then
      ZOOMCACHE="$CONVERT_TEMPDIR/${SRCFILE}.pnm"
      CACHEFILE="$CONVERT_TEMPDIR/pnmcache.lastfile"

      touch "$CACHEFILE"
      if [ ! "z`cat $CACHEFILE`" = "z$ZOOMCACHE" -o ! -e "$ZOOMCACHE" ] ; then
         rm -f "`cat $CACHEFILE`"                   # delete old cache
         logit "creating zoomcache for $SRCFILE"
         $TO_PNM "$SRCFILE" > "$ZOOMCACHE"
         echo "$ZOOMCACHE" > "$CACHEFILE"
      fi
      logit "using zoomcache for $SRCFILE"
      cat "$ZOOMCACHE" | \
        pnmcut -pad -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > "$ZOOMFILE"

   else 
      logit "not using zoomcache for $SRCFILE"
      logit "$TO_PNM $SRCFILE | pnmcut -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > $ZOOMFILE"
      $TO_PNM "$SRCFILE" | \
        pnmcut -pad -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > "$ZOOMFILE"
   fi
fi

# This is needed for rotating and aspect corretion
getresolution "$ZOOMFILE"


[ ! -d "$NEWDIR" ] && mkdir -p -v "$NEWDIR" | tee -a $CONVERT_LOGFILE

convert_imagetompeg "$ZOOMFILE" "$NEWFILE" "$FLIP"

if [ ! -f "$NEWFILE" ] ; then 
   logit "$DATE: ERROR Convert Image-to-MPEG2-STILL not successfull for : $IMAGEFILE"; 
   cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg" "$CONVERT_MPEGFILE" &>/dev/null
else
   logit "$DATE: successfull ...";

   if [ $(wc -c "$NEWFILE"|sed 's/^ *//'|cut -f1 -d" ") -gt 100 ] ; then
      logit "doing the copy to $CONVERT_MPEGFILE from $NEWFILE"
      rm -f "$CONVERT_MPEGFILE"
      cp -f "$NEWFILE" "$CONVERT_MPEGFILE" | tee -a "$CONVERT_LOGFILE"
   else
      logit "Error generated mpeg is smaller than 100 byte, so not used!!"
      cp -f "$CONVERT_TEMPDIR/vdr-image.black-screen.mpg" "$CONVERT_MPEGFILE" &>/dev/null
   fi
fi

rm -f "$ZOOMFILE"
